// threshVideo.cc
// g++ -I/usr/include/SDL -D_REENTRANT -L/usr/lib -lSDL -lSDL_image -lpthread -o threshVideo threshVideo.cc

using namespace std;
//#include<stdlib.h>
#include<iostream>
#include<fstream>
#include<cmath>
#include<SDL.h>
#include<SDL_image.h>
#include<string>
#include "convert.h"

/*
* Set the pixel at (x, y) to the given value
* NOTE: The surface must be locked before calling this!
*/
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
  *((Uint8 *)surface->pixels + y * surface->pitch + x) = pixel;
}
/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 * van file:/usr/share/doc/libsdl1.2-dev/docs/html/guidevideo.html
 */
Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

class ThresholdStream
{
 public:
  ThresholdStream(int hT) 
  {
    hardThreshold = hT;
    reset();
    double tauM = 4.0;
    double tauS = 2.0;
    double tauR = 20.0;
    tauMe = 1 / (exp(1/tauM));
    tauSe = 1 / (exp(1/tauS));
    tauRe = 1 / (exp(1/tauR));

  }
  // simpele versie, wordt moeilijker als thresholden met spike-neuron
  // (dus class nodig want waardes bewaren).
  int threshold(int input)
  {
    return(input < hardThreshold);
  }
  int spikeThreshold(int input)
  {
    //decays:
    uM = uM * tauMe;
    uS = uS * tauSe;
    uR = uR * tauRe; // doe ik dit ook nog steeds in spikeProp2 ?? -> ja zucht....
    //new input:
    if(input < 100)
      uM += 100 -input;
    // fire?
    if((uM + uS + uR) > hardThreshold)
    {
      uR -= hardThreshold;
      return(1);
    }
    else
      return(0);
  }
  void reset() 
  { 
    uR = 0.0;
    uS = 0.0;
    uM = 0.0;
  }
 private:
  int hardThreshold;
  double uM, uS, uR;
  double tauMe, tauSe, tauRe;
};

void videoToSpikeTrains(string fileName, int nrOfImages)
{
  string fileNameImage;
  Uint8 pixelValue;
  int thresholdedValue;
  ThresholdStream thresholdStream(50);
  int x,y,p,rust,v; 
  int countSpikes;
  
  string patfileName = "../invVideoPats/" + fileName + ".pat";
  ofstream patternStream(patfileName.c_str(), ofstream::out | ofstream::app | ofstream::trunc);

  SDL_Event  event;
  // load images (pgms)
  SDL_Surface *plaatje[nrOfImages];
  for(p=0; p<nrOfImages && (event.type != SDL_QUIT); p++)
  {
    fileNameImage = fileName;
    fileNameImage += ".000";
    if (p<9) // ugly...
      fileNameImage += "0";
    fileNameImage += stringify(p+1);
    plaatje[p] = IMG_Load(fileNameImage.c_str());
    SDL_LockSurface(plaatje[p]);
  }
  
  patternStream << (plaatje[0]->h * plaatje[0]->w) << endl;
  for(y=0; y<plaatje[0]->h; y++) // hight and width is the same in each image
    for(x=0; x<plaatje[0]->w; x++)
    {
      patternStream << "Input" << x << "-" << y << " ";
      // sigh....
      // first run to count the spikes:
      countSpikes = 0;
      for(p=0; p<nrOfImages; p++)
      {
        pixelValue = (Uint8)getpixel(plaatje[p], x, y);     
        countSpikes += thresholdStream.threshold(pixelValue);
      }
      patternStream << countSpikes << " ";
      thresholdStream.reset();
      // second run to save spike-times:
      countSpikes = 0;
      for(p=0; p<nrOfImages; p++)
      {
        pixelValue = (Uint8)getpixel(plaatje[p], x, y);     
        if(thresholdStream.threshold(pixelValue))
          patternStream << " " << p;
      }
      thresholdStream.reset();
      patternStream << endl;
    }
  patternStream.close();
}

void showThresholdedVideo(string patFileName, int nrOfImages, int scale, int thresholdValue, int nrOfCycles)
{
  int x, y, xs, ys, p, i;
  string imageFileName;
  Uint8 pixelValue;
  Uint8 putValue;
  
  // load first image to determine width and heigth of images
  SDL_Surface *image;
  imageFileName = patFileName + ".00001";
  image = IMG_Load(imageFileName.c_str());

  ThresholdStream* thresholdStream[image->w][image->h];
  for(y=0; y<image->h; y++)
    for(x=0; x<image->w; x++)
    {
      thresholdStream[x][y] = new ThresholdStream(thresholdValue);
    }

  // make graphics window
  SDL_Surface *screen;
  SDL_Init(SDL_INIT_VIDEO);
  screen = SDL_SetVideoMode(image->w * scale, image->h * scale, 8, 0);
  SDL_SetColors(screen, image->format->palette->colors, 0, image->format->palette->ncolors); 

  
  for(i=0; i<nrOfCycles; i++)
  {
    for(p=0; p<nrOfImages; p++)
    {
      if (p<nrOfImages)
      {
        imageFileName= patFileName + ".000";
        if (p<9) // wauw, hoe lelijk kan je het maken.
          imageFileName+= "0";
        imageFileName += stringify(p+1);
  //      cout << filenaam << endl;

        image = IMG_Load(imageFileName.c_str());
        SDL_LockSurface(image);
      }
      SDL_LockSurface(screen);
      for(y=0; y<image->h; y++)
        for(x=0; x<image->w; x++)
        {
          if (p<nrOfImages)
            pixelValue = ((Uint8)getpixel(image, x, y));     
          else 
            pixelValue = 255;
          if (thresholdValue)
          {
//            if ( (int) pixelValue < thresholdValue)
            if (thresholdStream[x][y]->threshold(pixelValue))
            {
              putValue = 255;
            }
            else
              putValue = 0;
          }
          else
            putValue = pixelValue;
          for(xs = scale*x; xs < scale* (x+1); xs++)
            for(ys = scale*y; ys < scale* (y+1); ys++)
              putpixel(screen, xs, ys, putValue);
        }
      SDL_UnlockSurface(screen);
      SDL_UpdateRect(screen, 0, 0, 0, 0);
      SDL_Delay(100);
    }
    SDL_Delay(1000);
    for(y=0; y<image->h; y++)
      for(x=0; x<image->w; x++)
        thresholdStream[x][y]->reset();
  } 
}

int main(int argc, char** argv)
{
  string naam = "Ben12";
  int nrOfImages = 10;
  int scale = 10;
  int threshold = 0;
  int nrOfTimes = 3; 
    
  if (argc == 2)
    naam = argv[1];

  showThresholdedVideo(naam, nrOfImages, scale, threshold, nrOfTimes);
  return(0);
}
