// threshold.cc
// g++ -I/usr/include/SDL -D_REENTRANT -L/usr/lib -lSDL -lSDL_image -lpthread -o thresholdNorm thresholdNorm.cc

#include<stdlib.h>
#include<iostream.h>
#include<fstream.h>
#include<SDL.h>
#include<SDL_image.h>
#include<string>
#include "convert.h"

/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 * van file:/usr/share/doc/libsdl1.2-dev/docs/html/guidevideo.html
 */
Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

class ThresholdStream
{
 public:
  ThresholdStream(int hT) 
  {
    hardThreshold = hT;
  }
  // simpele versie, wordt moeilijker als thresholden met spike-neuron
  // (dus class nodig want waardes bewaren).
  int threshold(int input)
  {
    return(input < hardThreshold);
  }
  void reset() 
  { // nog leeg
  }
 private:
  int hardThreshold;
};

int pijlNrOfSpikes(string fileName, int nrOfImages, int threshold)
{
  string fileNameImage;
  Uint8 pixelValue;
  int thresholdedValue;
  ThresholdStream thresholdStream(threshold);
  int x,y,p,rust,v; 
  int countSpikes;
  int countSpikesTot = 0;
  
  SDL_Event  event;
  // load images (pgms)
  SDL_Surface *plaatje[nrOfImages];
  for(p=0; p<nrOfImages && (event.type != SDL_QUIT); p++)
  {
    fileNameImage = fileName;
    fileNameImage += ".000";
    if (p<9) // ugly...
      fileNameImage += "0";
    fileNameImage += stringify(p+1);
    plaatje[p] = IMG_Load(fileNameImage.c_str());
    SDL_LockSurface(plaatje[p]);
  }
  
  for(y=0; y<plaatje[0]->h; y++) // hight and width is the same in each image
    for(x=0; x<plaatje[0]->w; x++)
    {
      thresholdStream.reset();
      for(p=0; p<nrOfImages; p++)
      {
        pixelValue = (Uint8)getpixel(plaatje[p], x, y);     
        if(thresholdStream.threshold(pixelValue))
          countSpikesTot++;
      }
      thresholdStream.reset();
    }
  return(countSpikesTot);
}

int findTreshold(string fileName, int nrOfImages, int desiredNrOfSpikes)
{
  int threshold = 50;
  int vorigeChThresh;
  int chThresh = 0;
  do
  {
    vorigeChThresh = chThresh;
    if(pijlNrOfSpikes(fileName, nrOfImages, threshold) > desiredNrOfSpikes)
    {
      chThresh = -1;
    }
    else
    {
      chThresh = 1;
    }
    threshold += chThresh;
  }
  while(chThresh == vorigeChThresh || !vorigeChThresh);
  if(chThresh == 1)
    return(threshold);
  else
    return(threshold+1);
}

void videoToSpikeTrains(string fileName, int nrOfImages)
{
  string fileNameImage;
  Uint8 pixelValue;
  int thresholdedValue;
  int threshold;
  int x,y,p,rust,v; 
  int countSpikes;
  int countSpikesTot = 0;
  
  threshold = findTreshold(fileName, nrOfImages, 2000);
  cout << " threshold = " << threshold;

  ThresholdStream thresholdStream(threshold);
  string patfileName = "../normInvVideoPats/" + fileName + ".pat";
  ofstream patternStream(patfileName.c_str(), ofstream::out | ofstream::app | ofstream::trunc);

  SDL_Event  event;
  // load images (pgms)
  SDL_Surface *plaatje[nrOfImages];
  for(p=0; p<nrOfImages && (event.type != SDL_QUIT); p++)
  {
    fileNameImage = fileName;
    fileNameImage += ".000";
    if (p<9) // ugly...
      fileNameImage += "0";
    fileNameImage += stringify(p+1);
    plaatje[p] = IMG_Load(fileNameImage.c_str());
    SDL_LockSurface(plaatje[p]);
  }
  
  patternStream << (plaatje[0]->h * plaatje[0]->w) << endl;
  for(y=0; y<plaatje[0]->h; y++) // hight and width is the same in each image
    for(x=0; x<plaatje[0]->w; x++)
    {
      patternStream << "Input" << x << "-" << y << " ";
      // sigh....
      // first run to count the spikes:
      countSpikes = 0;
      for(p=0; p<nrOfImages; p++)
      {
        pixelValue = (Uint8)getpixel(plaatje[p], x, y);     
        countSpikes += thresholdStream.threshold(pixelValue);
      }
      patternStream << countSpikes << " ";
      countSpikesTot += countSpikes;
      thresholdStream.reset();
      // second run to save spike-times:
      countSpikes = 0;
      for(p=0; p<nrOfImages; p++)
      {
        pixelValue = (Uint8)getpixel(plaatje[p], x, y);     
        if(thresholdStream.threshold(pixelValue))
          patternStream << " " << p;
      }
      thresholdStream.reset();
      patternStream << endl;
    }
  patternStream.close();
  cout << "  nrOfSpikes = " << countSpikesTot << endl;
}

  
int main(void)
{
  string videoName;
  int nrOfImages;
  ifstream aantallenStream("aantallen.data");
  while(!aantallenStream.eof())
  {
    aantallenStream >> videoName >> nrOfImages;
    cout << videoName << " " << nrOfImages; //  << endl;
    videoToSpikeTrains(videoName, nrOfImages);
  }

  return(0);
}
